//
// smpl_ADC4 : Analog-to-Digital Converter to read Photoresistor or IR Diode
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0
// J1  : jumper select photoresistor or Infrared Diode

// PA4 /ADC4 to GL5516/IR diode 

#include <stdio.h>
#include "Nano1X2Series.h"

void ADC_IRQHandler(void)
{
    uint32_t u32Flag;
    uint32_t u32ADCvalue;	

    // Get ADC conversion finish interrupt flag
    u32Flag = ADC_GET_INT_FLAG(ADC, ADC_ADF_INT);

    if(u32Flag & ADC_ADF_INT) {
        u32ADCvalue = ADC_GET_CONVERSION_DATA(ADC, 4);
        printf("GL5516 = %d\n",u32ADCvalue);
    }
    ADC_CLR_INT_FLAG(ADC, u32Flag);
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk | CLK_PWRCTL_LXT_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk | CLK_CLKSTATUS_LXT_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
    CLK_SetModuleClock(ADC_MODULE, CLK_CLKSEL1_ADC_S_HIRC, CLK_ADC_CLK_DIVIDER(1));
	
	  /* Enable IP clock */
		CLK_EnableModuleClock(ADC_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/	
    /* Set PA multi-function pins for ADC */
    SYS->PA_L_MFP &= ~SYS_PA_L_MFP_PA4_MFP_Msk;
    SYS->PA_L_MFP |= SYS_PA_L_MFP_PA4_MFP_ADC_CH4;

    /* Disable digital input path */
    PA->OFFD = PA->OFFD | (ADC_CH_4_MASK<< 16);
		
    /* Lock protected registers */
    SYS_LockReg();
}

void Init_ADC(void)
{
	  ADC_Open(ADC, ADC_INPUT_MODE_SINGLE_END, ADC_OPERATION_MODE_CONTINUOUS, ADC_CH_4_MASK );

    // Power on ADC
    ADC_POWER_ON(ADC);

    // Enable ADC ADC_IF interrupt
    ADC_EnableInt(ADC, ADC_ADF_INT);
    NVIC_EnableIRQ(ADC_IRQn);

    ADC_START_CONV(ADC);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int32_t main()
{	
    SYS_Init();	
	  Init_ADC();

    while(1) {
    }
}
