//
// Smpl_ADC : Analog-to-Digital Converter to read Gas Sensor
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0
// MQ7 : Vh=5V heating time = 60s, Vh=1.4V heating time = 90s

// PD12      to MQ7-H_EN; PD12=1, Vh=2.92V (=2.97V - 0.05V)
// PA5 /ADC5 to MQ7-OUT

#include <stdio.h>
#include "Nano1X2Series.h"

void ADC_IRQHandler(void)
{
    uint32_t u32Flag;
    uint32_t u32ADCvalue;	

    // Get ADC conversion finish interrupt flag
    u32Flag = ADC_GET_INT_FLAG(ADC, ADC_ADF_INT);

    if(u32Flag & ADC_ADF_INT) {
        u32ADCvalue = ADC_GET_CONVERSION_DATA(ADC, 5);
        printf("MQ7 = %d\n",u32ADCvalue);
    }
    ADC_CLR_INT_FLAG(ADC, u32Flag);
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk | CLK_PWRCTL_LXT_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk | CLK_CLKSTATUS_LXT_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
    CLK_SetModuleClock(ADC_MODULE, CLK_CLKSEL1_ADC_S_HIRC, CLK_ADC_CLK_DIVIDER(1));
	
	  /* Enable IP clock */
		CLK_EnableModuleClock(ADC_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/	
    /* Set PA multi-function pins for ADC */
    SYS->PA_L_MFP &= ~SYS_PA_L_MFP_PA5_MFP_Msk;
    SYS->PA_L_MFP |= SYS_PA_L_MFP_PA5_MFP_ADC_CH5;

    /* Disable digital input path */
    PA->OFFD = PA->OFFD | (ADC_CH_5_MASK<< 16);
		
    /* Lock protected registers */
    SYS_LockReg();
}

void Init_ADC(void)
{
	  ADC_Open(ADC, ADC_INPUT_MODE_SINGLE_END, ADC_OPERATION_MODE_CONTINUOUS, ADC_CH_5_MASK );

    // Power on ADC
    ADC_POWER_ON(ADC);

    // Enable ADC ADC_IF interrupt
    ADC_EnableInt(ADC, ADC_ADF_INT);
    NVIC_EnableIRQ(ADC_IRQn);

    ADC_START_CONV(ADC);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int32_t main()
{	
    SYS_Init();
	  Init_ADC();
	
	  GPIO_SetMode(PD, BIT12,GPIO_PMD_OUTPUT);
	  PD12=1; // MQ-7 H_EN 

    while(1) {
    }
}
