//
// Smpl_GPIO_IRQ : GPIO interrupt triggered by buttons
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0

// GPIOs
// PB12 to KEY1
// PB13 to KEY2
// PB14 to KEY3

#include <stdio.h>
#include "Nano1X2Series.h"

volatile uint8_t KEY1_Flag = 0;
volatile uint8_t KEY2_Flag = 0;
volatile uint8_t KEY3_Flag = 0;

void GPABC_IRQHandler(void)
{
    if (PB->ISRC & BIT12) {        // check if PB12 interrupt occurred
        PB->ISRC |= BIT12;         // clear PB12 interrupt status
			  KEY1_Flag=1;               // set a flag for PB12(KEY1)
    } else if (PB->ISRC & BIT13) { // check if PB13 interrupt occurred
        PB->ISRC |= BIT13;         // clear PB13 interrupt status  
        KEY2_Flag=1;	              // set a flag for PB13(KEY2)
    } else if (PB->ISRC & BIT14) { // check if PB14 interrupt occurred
        PB->ISRC |= BIT14;         // clear PB14 interrupt status  
        KEY3_Flag=1;	              // set a flag for PB14(KEY3)			
    } else {                      // else it is unexpected interrupts
        PB->ISRC = PB->ISRC;	      // clear all GPB pins
    }
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk | CLK_PWRCTL_LIRC_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk | CLK_CLKSTATUS_LIRC_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
	
	  /* Enable IP clock */

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/	
		
    /* Lock protected registers */
    SYS_LockReg();
}

void Init_KEY(void)
{
    GPIO_SetMode(PB, (BIT12 | BIT13 | BIT14), GPIO_PMD_INPUT);
    GPIO_ENABLE_PULL_UP(PB, (BIT12 | BIT13 | BIT14));	
    GPIO_EnableInt(PB, 12, GPIO_INT_FALLING);
    GPIO_EnableInt(PB, 13, GPIO_INT_FALLING);
    GPIO_EnableInt(PB, 14, GPIO_INT_FALLING);		
    NVIC_EnableIRQ(GPABC_IRQn);   
    GPIO_SET_DEBOUNCE_TIME(GPIO_DBCLKSRC_IRC10K, GPIO_DBCLKSEL_64);
    GPIO_ENABLE_DEBOUNCE(PB, (BIT12 | BIT13 | BIT14));	
}

/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main()
{	
    SYS_Init();	
	  Init_KEY();
	  printf("Testing KEY1/2/3 IRQ generation:\n");

    while(1) {	
			  if (KEY1_Flag) {
			        printf("KEY1/PB12 Interrupt!\n");
					    KEY1_Flag=0;
				}
			  if (KEY2_Flag) {
			        printf("KEY2/PB13 Interrupt!\n");
					    KEY2_Flag=0;
				}		
				if (KEY3_Flag) {
			        printf("KEY3/PB14 Interrupt!\n");
					    KEY3_Flag=0;
				}
		}
}
