//
// Smpl_MPU6050
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0

// I2C1 to OLED, LM75A, MPU6050
// PC10/I2C1-SCL to I2C-Slave SCL
// PC11/I2C1-SDA to I2C-Slave SDA

#include <stdio.h>
#include "Nano1X2Series.h"
#include "MPU6050.h"

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
    CLK_SetModuleClock(I2C1_MODULE, 0, 0);	
	
	  /* Enable IP clock */
		CLK_EnableModuleClock(I2C1_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Set multi function pin for I2C1 */
		SYS->PC_H_MFP &= ~(SYS_PC_H_MFP_PC10_MFP_Msk);
    SYS->PC_H_MFP |= (SYS_PC_H_MFP_PC10_MFP_I2C1_SCL);
		SYS->PC_H_MFP &= ~(SYS_PC_H_MFP_PC11_MFP_Msk);
    SYS->PC_H_MFP |= (SYS_PC_H_MFP_PC11_MFP_I2C1_SDA);		
		
    /* Lock protected registers */
    SYS_LockReg();
}

/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int32_t main()
{
	  int16_t accX, accY, accZ;
	  int16_t gyroX, gyroY, gyroZ;
	
    SYS_Init();	
	  I2C_Open(I2C1, 50000);	
    Init_MPU6050();
	
    while(1) {
			  accX = Read_MPU6050_AccX();
			  accY = Read_MPU6050_AccY();
			  accZ = Read_MPU6050_AccZ();
			  gyroX= Read_MPU6050_GyroX();
			  gyroY= Read_MPU6050_GyroY();
			  gyroZ= Read_MPU6050_GyroZ();			
			
			  printf("Acc=%6d,%6d,%6d, Gyro=%6d,%6d,%6d\n", accX, accY, accZ, gyroX, gyroY, gyroZ);
    }
}
