//
// Smpl_OLED 
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0

// I2C1 to OLED, LM75A, MPU6050
// PC10/I2C1-SCL to I2C-Slave SCL
// PC11/I2C1-SDA to I2C-Slave SDA

#include <stdio.h>
#include "Nano1X2Series.h"
#include "LY096BG30.h"

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
  	CLK_SetModuleClock(I2C1_MODULE, 0, 0);
	
	  /* Enable IP clock */
    CLK_EnableModuleClock(I2C1_MODULE);	
		
    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set multi function pin for I2C1 */
		SYS->PC_H_MFP &= ~(SYS_PC_H_MFP_PC10_MFP_Msk);
    SYS->PC_H_MFP |= (SYS_PC_H_MFP_PC10_MFP_I2C1_SCL);
		SYS->PC_H_MFP &= ~(SYS_PC_H_MFP_PC11_MFP_Msk);
    SYS->PC_H_MFP |= (SYS_PC_H_MFP_PC11_MFP_I2C1_SDA);	
		
    /* Lock protected registers */
    SYS_LockReg();
}

/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main()
{
    SYS_Init();
	  I2C_Open(I2C1, 100000);	//set I2C1 running at 100KHz
    Init_LCD();
	  clear_LCD();
	
    print_Line(0, "NuTiny - Nano102");
	  print_Line(1, "Cortex-M0 @32MHz");
	  print_Line(2, "Ultra Low Power ");
	  print_Line(3, "0.96 OLED 128x64");	

    print_C(0, 0, 'N');
    print_C(0, 1, 'C');
    print_C(0, 2, 'U');
    print_C(0, 3, ' ');
}

