//
// Smpl_PWM : Pulse Width Modulation output to drive Buzzer
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0

// PD9 /PWM0_CH3 to Buzzer

#include <stdio.h>
#include "Nano1X2Series.h"
#include "pwm.h"

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk | CLK_PWRCTL_LXT_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk | CLK_CLKSTATUS_LXT_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
    CLK_SetModuleClock(PWM0_CH23_MODULE, CLK_CLKSEL1_PWM0_CH23_S_HIRC, 0);	
	
	  /* Enable IP clock */
		CLK_EnableModuleClock(PWM0_CH23_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/	
    /* Set PD multi-function pins for PWM */
    SYS->PD_H_MFP = SYS->PD_H_MFP & ~(SYS_PD_H_MFP_PD9_MFP_Msk) | SYS_PD_H_MFP_PD9_MFP_PWM0_CH3;
	
    /* Lock protected registers */
    SYS_LockReg();
}

void Init_PWM(void) 
{
    // PWM0 CH3 frequency is 1KHz, duty 50%,
    PWM_ConfigOutputChannel(PWM0, 3, 1000, 50);
    // Enable output of all PWM channels
    PWM_EnableOutput(PWM0, BIT3);
    // Start
    PWM_Start(PWM0, BIT3);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int32_t main()
{	
    SYS_Init();	
	  Init_PWM();

    while(1) {
    }
}
