//
// Smpl_PWM_music : Pulse Width Modulation output to drive Buzzer
//
// MCU : Nano102SC2AN
// EVB : NuMaker TRIO

// PD9 /PWM0_CH3 to Buzzer

#include <stdio.h>
#include "Nano1X2Series.h"
#include "pwm.h"
#include "Note_Freq.h"

#define  P125ms 125000
#define  P250ms 250000
#define  P500ms 500000
#define  P1S   1000000

  uint16_t music[72] = {
	E6 ,D6,E6 ,D6,E6 ,B5 ,D6 ,C6 ,A5 ,A5 , 0 , 0 ,
	C5 ,E5 ,A5 ,B5 ,B5 , 0 ,C5 ,A5 ,B5 ,C6 ,C6 , 0 ,
	E6 ,D6,E6 ,D6,E6 ,B5 ,D6 ,C6 ,A5 ,A5 , 0 , 0 ,
	C5 ,E5 ,A5 ,B5 ,B5 , 0 ,E5 ,C6 ,B5 ,A5 ,A5 , 0 ,
	B5 ,C6 ,D6 ,E6 ,E6 , 0 ,G5 ,F6 ,E6 ,D6 ,D6 , 0 ,
	F5 ,E6 ,D6 ,C6 ,C6 , 0 ,E5 ,D6 ,C6 ,B5 ,B5 , 0 };
	
  uint32_t pitch[72] = {
	P250ms, P250ms, P250ms, P250ms,	P250ms, P250ms, P250ms, P250ms,	P250ms, P250ms, P250ms, P250ms,
	P250ms, P250ms, P250ms, P250ms,	P250ms, P250ms, P250ms, P250ms,	P250ms, P250ms, P250ms, P250ms,
	P250ms, P250ms, P250ms, P250ms,	P250ms, P250ms, P250ms, P250ms,	P250ms, P250ms, P250ms, P250ms,
	P500ms, P500ms, P500ms, P500ms,	P500ms, P500ms, P500ms, P500ms,	P500ms, P500ms, P500ms, P500ms,
	P500ms, P500ms, P500ms, P500ms,	P500ms, P500ms, P500ms, P500ms,	P500ms, P500ms, P500ms, P500ms,
	P500ms, P500ms, P500ms, P500ms,	P500ms, P500ms, P500ms, P500ms,	P500ms, P500ms, P500ms, P500ms
	};
	
void SYS_Init(void)
{
    SYS_UnlockReg();   // Unlock protected registers

    // Set Chip clock source & Core clock
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk);
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk);
	  CLK_SetCoreClock(32000000);
	
    // Set IP clock source & enable	
    CLK_SetModuleClock(PWM0_CH23_MODULE, CLK_CLKSEL1_PWM0_CH23_S_HIRC, 0);
	  CLK_EnableModuleClock(PWM0_CH23_MODULE);

    //---------------------------------------------------------------------------------------------------------
    // Init I/O Multi-function                                                                                 
    //---------------------------------------------------------------------------------------------------------
    // Set PD multi-function pins for PWM
    SYS->PD_H_MFP = (SYS->PD_H_MFP & ~SYS_PD_H_MFP_PD9_MFP_Msk) | SYS_PD_H_MFP_PD9_MFP_PWM0_CH3;
	
    SYS_LockReg();    // Lock protected registers
}

void Init_PWM(void) 
{
    PWM_EnableOutput(PWM0, BIT3); // enable PWM0 Channel 3
    PWM_Start(PWM0, BIT3);
}

int32_t main()
{	
	uint8_t i;
	
    SYS_Init();	
	  Init_PWM();

	  for (i=0; i<72; i++) {
			if (music[i] > 0)
			{
				PWM_ConfigOutputChannel(PWM0, 3, music[i], 20);
				PWM_EnableOutput(PWM0, BIT3);
			}
		CLK_SysTickDelay(pitch[i]);
			PWM_DisableOutput(PWM0, BIT3);
    }
		PWM_DisableOutput(PWM0, BIT3); // disable PWM0 Channel 3
}
