//
// Smpl_RTC : Real Timer Clock
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0

#include <stdio.h>
#include "Nano1X2Series.h"

volatile uint32_t  g_u32TICK = 0;

void RTC_IRQHandler(void)
{
    S_RTC_TIME_DATA_T sCurTime;

    if ( (RTC->RIER & RTC_RIER_TIER_Msk) && (RTC->RIIR & RTC_RIIR_TIF_Msk) ) {// tick interrupt occurred
        RTC->RIIR = 0x2;

    RTC_GetDateAndTime(&sCurTime);  // Get the current time

	  printf("Count:%4d,Time: %4d/%2d/%2d %2d:%2d:%2d\n", g_u32TICK,
		sCurTime.u32Year,sCurTime.u32Month,sCurTime.u32Day,sCurTime.u32Hour,sCurTime.u32Minute,sCurTime.u32Second);
	
    g_u32TICK++;
    }
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk | CLK_PWRCTL_LXT_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk | CLK_CLKSTATUS_LXT_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
    CLK_SetModuleClock(RTC_MODULE, 0, 0);	
	
	  /* Enable IP clock */
		CLK_EnableModuleClock(RTC_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/	
		
    /* Lock protected registers */
    SYS_LockReg();
}

void Init_RTC(void)
{
		S_RTC_TIME_DATA_T sInitTime;

    /* Time Setting */
    sInitTime.u32Year       = 2014;
    sInitTime.u32Month      = 7;
    sInitTime.u32Day        = 3;
    sInitTime.u32Hour       = 13;
    sInitTime.u32Minute     = 30;
    sInitTime.u32Second     = 0;
    sInitTime.u32DayOfWeek  = RTC_THURSDAY;
    sInitTime.u32TimeScale  = RTC_CLOCK_24;

	  RTC_Open(&sInitTime);

    /* Set Tick Period */
    RTC_SetTickPeriod(RTC_TICK_1_SEC);

    /* Enable RTC Tick Interrupt */
    RTC_EnableInt(RTC_RIER_TIER_Msk);
    NVIC_EnableIRQ(RTC_IRQn);
}
/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int32_t main()
{	
    SYS_Init();	
	  Init_RTC();

    while(1) {
    }
}
