//
// Smpl_TMR0 : Timer0
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0

#include <stdio.h>
#include "Nano1X2Series.h"
volatile uint32_t timer_count =0;

void TMR0_IRQHandler(void)
{
	  timer_count++;
    printf("Timer0 = %d\n", timer_count);
    // clear timer interrupt flag
    TIMER_ClearIntFlag(TIMER0);
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable Internal RC (12 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk);

    /* Set HCLK clock source */	
		//CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_HCLK_CLK_DIVIDER(1));
	  CLK_SetCoreClock(32000000);
	
    /* Select IP clock source */
    CLK_SetModuleClock(TMR0_MODULE, CLK_CLKSEL1_TMR0_S_HIRC, CLK_TMR0_CLK_DIVIDER(1));	
	
	  /* Enable IP clock */
		CLK_EnableModuleClock(TMR0_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/	
		
    /* Lock protected registers */
    SYS_LockReg();
}

void Init_Timer0(void)
{
    // Give a dummy target frequency here. Will over write capture resolution with macro
    TIMER_Open(TIMER0, TIMER_PERIODIC_MODE, 50); // 50Hz

    // Enable timer interrupt
    TIMER_EnableInt(TIMER0);
    NVIC_EnableIRQ(TMR0_IRQn);

    // Start Timer 0
    TIMER_Start(TIMER0);
}
/*---------------------------------------------------------------------------------------------------------*/
/* Main Function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int32_t main()
{
	  uint16_t tmp16;
	  int8_t  msb, lsb;
	
    SYS_Init();	
	  Init_Timer0();

    while(1) {
    }
}
