//
// smpl_Nuwarocks : use all sensors of Nuwarocks v1.0
//
// MCU : Nano102SC2AN
// EVB : NUWAROCKS v1.0

// I2C1 to OLED, LM75A, MPU6050
// PC10/I2C1-SCL to I2C-Slave SCL
// PC11/I2C1-SDA to I2C-Slave SDA

// GPIOs
// PC4  to BT Reset
// PC6  to WiFi Reset
// PC12 to WiFi Sleep
// PC13 to MPU6050 INT
// PB12 to KEY1
// PB13 to KEY2
// PB14 to KEY3

// PD9 /PWM0_CH3 to Buzzer

// PD11 to DHT11-IN
// PD12 to H-EN (enable MQ7)

// PA4 /ADC4 to GL5516/IR diode
// PA5 /ADC5 to MQ7-Out

// UART0 to BT   module
// PA13/UART0-RX to BT-TX
// PA12/UART0-TX to BT-RX

// UART1 to WiFi module
// PC7 /UART1-RX to WiFi-TX
// PC8 /UART1-TX to WiFi-RX

#include <stdio.h>
#include "Nano1X2Series.h"
#include "DHT11.h"
#include "OLED_SSD1306Z.h"

volatile uint16_t time_DHT11[40];
volatile uint32_t capture_count =0;
volatile uint16_t DHT11_D[2];

void TMR0_IRQHandler(void)
{
    time_DHT11[capture_count] = TIMER_GetCaptureData(TIMER0);
		capture_count++;
    TIMER_ClearCaptureIntFlag(TIMER0);
}

void SYS_Init(void)
{
    SYS_UnlockReg();   // Unlock protected registers

    // select Chip clock source & set core clock
    CLK_EnableXtalRC(CLK_PWRCTL_HIRC_EN_Msk | CLK_PWRCTL_LXT_EN_Msk);
    CLK_WaitClockReady(CLK_CLKSTATUS_HIRC_STB_Msk | CLK_CLKSTATUS_LXT_STB_Msk);
	  CLK_SetCoreClock(32000000);
	
    // Select IP clock source & enable module	
  	CLK_SetModuleClock(TMR0_MODULE,  CLK_CLKSEL1_TMR0_S_HIRC, CLK_TMR0_CLK_DIVIDER(1));		
		CLK_EnableModuleClock(TMR0_MODULE);
	
    // Init I/O Multi-function

    SYS_LockReg(); // Lock protected registers
}

int32_t main()
{
		SYS_Init();
	
		Init_DHT11();
	 
    while(1) {
		   Read_DHT11(DHT11_D);
			 printf("Humidity    = %x\n", DHT11_D[0]);
			 printf("Temperature = %d\n", DHT11_D[1]);		
    }
}
